@@ -1,10 +1,11 @@
-from .collect import CollectStep
+from .collect import CollectStep, ParallelCollectStep
 from .factory import KafkaConsumerStrategyFactory
 from .filter import FilterStep
 from .transform import ParallelTransformStep, TransformStep
 
 __all__ = [
     "CollectStep",
+    "ParallelCollectStep",
     "FilterStep",
     "ParallelTransformStep",
     "TransformStep",
