@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from datetime import datetime
-from typing import Generic, Optional, TypeVar
+from typing import Generic, TypeVar
 
 
 @dataclass(order=True, unsafe_hash=True)
@@ -32,13 +32,12 @@ class Message(Generic[TPayload]):
     Represents a single message within a partition.
     """
 
-    __slots__ = ["partition", "offset", "payload", "timestamp", "next_offset"]
+    __slots__ = ["partition", "offset", "payload", "timestamp"]
 
     partition: Partition
     offset: int
     payload: TPayload
     timestamp: datetime
-    next_offset: int
 
     def __init__(
         self,
@@ -46,16 +45,11 @@ class Message(Generic[TPayload]):
         offset: int,
         payload: TPayload,
         timestamp: datetime,
-        next_offset: Optional[int] = None,
     ) -> None:
-        if next_offset is None:
-            next_offset = offset + 1
-
         self.partition = partition
         self.offset = offset
         self.payload = payload
         self.timestamp = timestamp
-        self.next_offset = next_offset
 
     def __repr__(self) -> str:
         # XXX: Field values can't be excluded from ``__repr__`` with
@@ -64,6 +58,10 @@ class Message(Generic[TPayload]):
         # would conflict with the instance slot names, causing an error.
         return f"{type(self).__name__}(partition={self.partition!r}, offset={self.offset!r})"
 
+    @property
+    def next_offset(self) -> int:
+        return self.offset + 1
+
 
 @dataclass(frozen=True)
 class Position:
